#!/bin/bash
# ------------------------------------------------------------------------------
# WRFOTRON v 2.0b
# Christoph Knote (LMU Munich, Germany)
# 06/2016
# christoph.knote@lmu.de
# ------------------------------------------------------------------------------

# path to the WRFotron installation
chainDir=/nobackup/${USER}/wrfotron

# --- Executable locations ---

# WPS installation directory
WPSDir=/nobackup/cemac/software/build/WRFChem/4.2/2/intel-19.0.4-openmpi-3.1.4/WRFChem4.2/WPS-4.2
# WRF installation directory
WRFDir=/nobackup/cemac/software/build/WRFChem/4.2/2/intel-19.0.4-openmpi-3.1.4/WRFChem4.2/WRFMeteo4.2
# --- Input data settings ---

# path to geogrid input data
geogDir=/nobackup/WRFChem/WPSGeog4

# meteo input
# initial and boundary meteorological data
metDir=/nobackup/WRFChem/initial_boundary_meteo_ecmwf
# Vtable for the chosen meteo input
metVtableFile=/nobackup/WRFChem/WPS4.2/ungrib/Variable_Tables/Vtable.ECMWF
# time increment in hours
metInc=6
# full path to a met input file - you can use any "%<>" abbreviations known
# to the "date" command
metFilePattern="${WRF_GFS_METEO_PATH}/GF%Y%m%d%H"
# example:
# "/glade/p/rda/data/ds083.2/grib2/%Y/%Y.%m/fnl_%Y%m%d_%H_00.grib2"

# --- Pre/Postprocessing settings ---

# prepararation script
preScriptPath=NONEXISTENT.bash

# postprocessing scripts (arbitrary)
postScriptPath=NONEXISTENT.bash
# postprocessing scripts (actions for each wrfout file)
postPerFileScriptPath=postprocessing.py

# --- Working directories ---

projectTag=simulation_WRFChem4.2_augsburg

# where the WRF will be run (some fast, large disk like "scratch" or similar)
workDir=/nobackup/${USER}/${projectTag}/run

# where the unprocessed WRF output will be stored
stagingRootDir=/nobackup/${USER}/${projectTag}/staging
# where the WRF output will be stored
archiveRootDir=/nobackup/${USER}/${projectTag}/output
# where the WRF restart files will be stored
restartRootDir=/nobackup/${USER}/${projectTag}/restart

# remove run directory after run is finished?
removeRunDir=false

# --- MPI settings ---

mpiCommandPre=mpirun
mpiCommandMain="mpirun -bind-to none"
mpiCommandReal=mpirun

# --- Batch system ---

# argument to submit a job in a held state
batchHoldArgument="-h"
# command to release a held job
batchReleaseCommand="qrls"
# command to submit jobs to the queueing system
batchSubmitCommand="qsub"
# dependency argument for chaining runs upon submission
batchDepArgument="-hold_jid __id__"
# sed command ("used as s/__command/\1/") to retrieve job run PID upon
# submission with $batchSubmitCommand
batchPidSedCommand="Your job \([0-9]\{1,10\}\) (.*) has been submitted"
# character to feed in job script: "<" for Christoph
batchChar=""

# --- Chemistry ---

withChemistry=true

# WRF-Chem installation directory
WRFChemDir=/nobackup/cemac/software/build/WRFChem/4.2/2/intel-19.0.4-openmpi-3.1.4/WRFChem4.2/WRFChem4.2

# megan_bio_emiss installation directory
WRFMEGANdir=/nobackup/cemac/software/build/WRFChem/4.2/2/intel-19.0.4-openmpi-3.1.4/WRFChem4.2/megan
# mozbc installation directory
WRFMOZARTdir=/nobackup/cemac/software/build/WRFChem/4.2/2/intel-19.0.4-openmpi-3.1.4/WRFChem4.2/mozbc
# wesley/exocoldens installation directory
WRFmztoolsdir=/nobackup/cemac/software/build/WRFChem/4.2/2/intel-19.0.4-openmpi-3.1.4/WRFChem4.2/wes-coldens
# anthro_emiss installation directory
WRFanthrodir=/nobackup/cemac/software/build/WRFChem/4.2/2/intel-19.0.4-openmpi-3.1.4/WRFChem4.2/anthro_emis
# fire_emis installation directory
WRFfiredir=/nobackup/cemac/software/build/WRFChem/4.2/2/intel-19.0.4-openmpi-3.1.4/WRFChem4.2/finn/src

# path to MEGAN input data
MEGANdir=/nobackup/WRFChem/emissions/MEGAN

# use anthro_emiss or predefined files?
emissUseAnthroEmiss=true
# raw emission input - the files you read in with anthro_emiss
emissDir=/nobackup/WRFChem/emissions/EDGAR-HTAP2_MEIC2015/MOZART
# emission conversion script for anthro_emis - must match emissions in emissDir
emissInpFile=emis_edgarhtap2-meic2015_mozmos.inp
# year the emissions are valid for (for offset calculation)
emissYear=2010
# diurnal cycle code
WRFemitdir=/nobackup/WRFChem/WRF_UoM_EMIT

# FINN fires
fireDir=/nobackup/WRFChem/emissions/FINN/
fireFilePattern="/nobackup/WRFChem/emissions/FINN/FINNv1.5_%Y.MOZ4"
fireInpFile=fire_emis.mozm.inp

# boundary condition input
MOZARTdir=/nobackup/WRFChem/initial_boundary_chem_mz4
chembcFilePattern="/nobackup/WRFChem/initial_boundary_chem_mz4/MZ%Y%m"
chembcInpFile=mozbc.inp

# TUV photolysis option 4 data file
TUVDataPath="/nobackup/WRFChem/tuv_files/TUV.phot.bz2"

