#!/bin/bash

module load python

export MPLBACKEND="agg"

scriptDir=/home/m/met-wrf-chem/WRFotron/blueprints/operational/pp/plotting/maps
wrfDataPathPattern="/project/meteo/data/wrf/output/operational/wrfout_d{domain:s}_%Y-%m-%d_%H:%M:%S"
tmpWorkDir="/project/meteo/scratch/met-wrf-chem/wrf_fcst_movie_processing"
plotPathPattern="${tmpWorkDir}/{plot:s}_%Y%m%d%H.png"
webDir="/project/meteo/homepages/quicklooks/wrf/maps/"
dateToProcess="today"
fcstIntervalHours=1
fcstRangeHours=72
nSecondsWait=2

mkdir -p ${tmpWorkDir}
rm -f ${tmpWorkDir}/*

cd $scriptDir

python plot.py ${wrfDataPathPattern} $(date -u --date="${dateToProcess}" "+%Y-%m-%d") ${fcstIntervalHours} ${fcstRangeHours} ${plotPathPattern}

cd ${tmpWorkDir}

# once ffmpeg gets updated (to something > 2 Nov 2018), we can add still frames at the end with this -vf filter:
#tpad=stop_mode=clone:stop_duration=2

# see here for why we need vf argument https://stackoverflow.com/questions/20847674/ffmpeg-libx264-height-not-divisible-by-2
frameRate=3
for typ in 500hPa_Europe sfc_SouthernGermany slp_Europe
do
  # super ugly hack: copy last image seconds*framerate times to another name that ensures they are at the end (using big number as date here)
  # to make video "hold" on last image for 'seconds'
  allFiles=( $(ls -r1 ./${typ}_*.png) )
  lastFile=${allFiles[0]}
  for iter in $(seq 0 ${nSecondsWait})
  do
    for frm in $(seq 0 ${frameRate})
    do 
      let bignum='300000000000+iter*frm'
      cp ${lastFile} ./${typ}_${bignum}.png
    done
  done
  ffmpeg -y -r ${frameRate} -pattern_type glob  -i "./${typ}_*.png" -vcodec libx264 -crf 18 -preset slow -pix_fmt yuv420p -vf "pad=ceil(iw/2)*2:ceil(ih/2)*2" current_${typ}.mp4 > /dev/null
done


for anHour in $(seq 0 ${fcstIntervalHours} ${fcstRangeHours})
do
  # get path to plots which we created in tmpdir
  plotPath=$(date -u --date="${dateToProcess} 0 UTC +${anHour} hours" "+${plotPathPattern}")
  # replace "type" placeholder with wildcard to match all
  allPlotPath=${plotPath/\{plot:s\}/\*}
  
  # where to put them on the web
  plotWebDir=$(date -u --date="${dateToProcess} 0 UTC +${anHour} hours" "+${webDir}/%Y/%m/%d")
  mkdir -p $plotWebDir
  
  mv ${allPlotPath} ${plotWebDir}
done

# movies go to top folder
mv current_*.mp4 ${webDir} 
