#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Tue Oct  9 12:13:31 2018
Creating colormap
@author: Markus.Laufmann
"""

import numpy as np
import os
import gzip
from netCDF4 import Dataset as data
import shutil


def str_to_bool(s):
    if s == 'True':
         return True
    elif s == 'False':
         return False
    else:
         raise ValueError # evil ValueError that doesn't tell you what the wrong value was
        
def make_cmap(colors, position=None, bit=False):
    '''
    make_cmap takes a list of tuples which contain RGB values. The RGB
    values may either be in 8-bit [0 to 255] (in which bit must be set to
    True when called) or arithmetic [0 to 1] (default). make_cmap returns
    a cmap with equally spaced colors.
    Arrange your tuples so that the first color is the lowest value for the
    colorbar and the last is the highest.
    position contains values from 0 to 1 to dictate the location of each color.
    
    From http://schubert.atmos.colostate.edu/~cslocum/custom_cmap.html
    '''
    import matplotlib as pl
    bit_rgb = np.linspace(0,1,256)
    if position == None:
        position = np.linspace(0,1,len(colors))
    else:
        if len(position) != len(colors):
            sys.exit("position length must be the same as colors")
        elif position[0] != 0 or position[-1] != 1:
            sys.exit("position must start with 0 and end with 1")
    if bit:
        for i in range(len(colors)):
            colors[i] = (bit_rgb[colors[i][0]],
                         bit_rgb[colors[i][1]],
                         bit_rgb[colors[i][2]])
    cdict = {'red':[], 'green':[], 'blue':[]}
    for pos, color in zip(position, colors):
        cdict['red'].append((pos, color[0], color[0]))
        cdict['green'].append((pos, color[1], color[1]))
        cdict['blue'].append((pos, color[2], color[2]))
    cmap = pl.colors.LinearSegmentedColormap('my_colormap',cdict,128)
    return cmap


def make_cmap_equal(colors, position=None, bit=False, equal=False,hist=None,bins=128):
    '''
    make_cmap takes a list of tuples which contain RGB values. The RGB
    values may either be in 8-bit [0 to 255] (in which bit must be set to
    True when called) or arithmetic [0 to 1] (default). make_cmap returns
    a cmap with equally spaced colors.
    Arrange your tuples so that the first color is the lowest value for the
    colorbar and the last is the highest.
    position contains values from 0 to 1 to dictate the location of each color.
    
    From http://schubert.atmos.colostate.edu/~cslocum/custom_cmap.html
    Edited and extended for equalization
    Methode: 
    '''
    import matplotlib as mpl
    import matplotlib.pyplot as pl
    bit_rgb = np.linspace(0,1,256)
    if position == None:
        position = np.linspace(0,1,len(colors))
    else:
        if len(position) != len(colors):
            sys.exit("position length must be the same as colors")
        elif position[0] != 0 or position[-1] != 1:
            sys.exit("position must start with 0 and end with 1")
    if bit:
        for i in range(len(colors)):
            colors[i] = (bit_rgb[colors[i][0]],
                         bit_rgb[colors[i][1]],
                         bit_rgb[colors[i][2]])
    cdict = {'red':[], 'green':[], 'blue':[]}
    if equal:
        thshold=np.sum(hist[0])/len(colors) #Festlegen der Grenze für jede Farbe 
        histogram=np.zeros((len(colors)))
        if hist==None:
            sys.exit("Histogram data needed")
        tmp_sum=0.
        tmp_added=0
        pos=np.linspace(0,1,bins)
        pos_id=0
        for i in range(1,len(colors)-1):
            if i==1:
                cdict['red'].append((pos[pos_id], colors[i-1][0], colors[i-1][0]))
                cdict['green'].append((pos[pos_id], colors[i-1][1], colors[i-1][1]))
                cdict['blue'].append((pos[pos_id], colors[i-1][2], colors[i-1][2]))
                thshold=(np.sum(hist[0])-hist[0][0])/(len(colors)-1)
                pos_id+=1
            #Aufsummieren bis Limit überschritten
            while tmp_sum<thshold:
                p=hist[0][pos_id]
                if tmp_sum+p<thshold:
                    cdict['red'].append((pos[pos_id], colors[i][0], colors[i][0]))
                    cdict['green'].append((pos[pos_id], colors[i][1], colors[i][1]))
                    cdict['blue'].append((pos[pos_id], colors[i][2], colors[i][2]))
                else:
                    tmp=(-tmp_sum+p+thshold)/p
                    if tmp<0.5:
                        cdict['red'].append((pos[pos_id], colors[i+1][0], colors[i+1][0]))
                        cdict['green'].append((pos[pos_id], colors[i+1][1], colors[i+1][1]))
                        cdict['blue'].append((pos[pos_id], colors[i+1][2], colors[i+1][2]))
                    else:
                        cdict['red'].append((pos[pos_id], colors[i][0], colors[i][0]))
                        cdict['green'].append((pos[pos_id], colors[i][1], colors[i][1]))
                        cdict['blue'].append((pos[pos_id], colors[i][2], colors[i][2]))
                tmp_sum+=p
                pos_id+=1
            histogram[i]=tmp_sum-p
            tmp_sum-=thshold
            if i==len(colors)-2:
                istrich=len(colors)-1
                for n in range(pos_id,bins):
                    cdict['red'].append((pos[n], colors[istrich][0], colors[istrich][0]))
                    cdict['green'].append((pos[n], colors[istrich][1], colors[istrich][1]))
                    cdict['blue'].append((pos[n], colors[istrich][2], colors[istrich][2]))
                    
        cmap = mpl.colors.LinearSegmentedColormap('my_colormap',cdict,bins)           
    else:
        for pos, color in zip(position, colors):
            cdict['red'].append((pos, color[0], color[0]))
            cdict['green'].append((pos, color[1], color[1]))
            cdict['blue'].append((pos, color[2], color[2]))

        cmap = pl.colors.LinearSegmentedColormap('my_colormap',cdict,bins)
    return cmap

def create_cmap(fname,bins=128):
    """
    Creates a simple colormap for plotting data
    Arguments:
    - fname: Location of the main colormap data
    - bins=128: Every color in the main cmap is selected for certain bins, total number of bins for it 
    """
    cmapdata=np.loadtxt(fname)
    cmap=make_cmap(cmapdata.T)
    return cmap


def create_cmap_equal(fname,data,bins=128,vmin=4.0,vmax=7.0):
    """
    Creates a colormap with histogram equalization for better contrast for plotting data
    Arguments:
    - fname: Location of the main colormap data
    - bins=128: Every color in the main cmap is selected for certain bins, total number of bins for it 
    - vmin=4.0: Minimum value for histogram
    - vmax=7.0: Maximum value for histogram
    """
    import matplotlib.pyplot as pl
    cmapdata=np.loadtxt(fname)
    hist=pl.hist(data.flatten(),bins,range=(vmin,vmax))
    cmap=make_cmap_equal(cmapdata.T,equal=True,bins=bins,hist=hist)
    return cmap

def use_now_asdate(data_maindir,ufs=False,use_yd=False,device='lidar',suffix=''):
    """
    Returns the location of device datafiles of today/yesterday
    Arguments:
    - data_maindir: main directory of the data files, below this maindir the folder structure should be e.g. YYYY/MM/mim_YYMMDD.nc for lidar data
    - ufs=False: plot lidar data from UFS Schneefernerhaus at the Zugspitze yes/no
    - use_yd=False: Plot device data from today (False) or yesterday (True) 
    - device='lidar': Change the device type to get the correct file location. Currently supported 'lidar','miraMACS',soon: 'HatPro'
    """
    import datetime as dt
    if use_yd:
        now=dt.datetime.utcnow()-dt.timedelta(days=1)
    else:
        now=dt.datetime.utcnow()
    if now.month<10:
        mo_corr='0'
    else:
        mo_corr=''
    if now.day<10:
        d_corr='0'
    else:
        d_corr=''
    if device=='lidar':
        yymm=str(now.year)+'/'+mo_corr+str(now.month)
        datum=str(now.year)[2:]+mo_corr+str(now.month)+d_corr+str(now.day)
        if ufs:
            lidarpath=data_maindir+yymm+'/ufs_'+datum+'.nc'
        else:
            lidarpath=data_maindir+yymm+'/mim_'+datum+'.nc'
        return lidarpath,datum
    elif device=='miraMACS':
        datum1=str(now.year)+mo_corr+str(now.month)+d_corr+str(now.day)
        datum2=str(now.year)[2:]+mo_corr+str(now.month)+d_corr+str(now.day)
        cloudpath=data_maindir+datum1+'_0000.mmclx'
        return cloudpath,datum2
    elif device=='HatPro':
        datum1='Y'+str(now.year)+'/M'+mo_corr+str(now.month)+'/D'+d_corr+str(now.day)+'/'+suffix
        datum2=str(now.year)[2:]+mo_corr+str(now.month)+d_corr+str(now.day)
        hatpropath=[data_maindir+datum1+datum2+'.TPC.NC',data_maindir+datum1+datum2+'.HPC.NC']
        return hatpropath,datum2


def use_custdate(custdate,data_maindir,ufs=False,device='lidar'):
    """
    Returns the location of lidar datafiles from any day
    Arguments:
    - custdate: any date in the format YYMMDD as string or as int 
    - data_maindir: main directory of the data files, below this maindir the folder structure should be YYYY/MM/mim_YYMMDD.nc
    - ufs=False: plot lidar data from UFS Schneefernerhaus at the Zugspitze yes/no
    - device='lidar': Change the device type to get the correct file location. Currently supported 'lidar','miraMACS',soon: 'HatPro'
    """
    if type(custdate)==int:
        date=str(custdate)
    else:
        date=custdate
    #If date formatted like this '51201' (01.12.2005)
    if len(date)==5:
        date='0'+date
    #Device-selection and creating the correct paths 
    if device=='lidar':
        yymm='20'+date[0:2]+'/'+date[2:4]
        datum=date
        if ufs:
        	lidarpath=data_maindir+yymm+'/ufs_'+datum+'.nc'
        else:
        	lidarpath=data_maindir+yymm+'/mim_'+datum+'.nc'
        return lidarpath,datum
    elif device=='miraMACS':
        datum1='20'+date
        cloudpath=data_maindir+datum1+'_0000.mmclx'
        return cloudpath,date
    elif device=='HatPro':
        datum1='Y20'+date[0:2]+'/M'+date[2:4]+'/D'+date[4:6]+'/'
        #List with NetCDF-Files for temperature and humidity profiles
        hatpropath=[data_maindir+datum1+date+'.TPC.NC',data_maindir+datum1+date+'.HPC.NC'] 
        return hatpropath,date
    
def get_sunrise_and_sunset(datum,lat=48.148,lon=11.573,elev=515,ret_format='str'):
    """
    Gibt Sonnenauf und untergang fuer einen beliebigen Ort als Strings zurueck.
    Argumente:
        datum: String mit dem Datum im Format YYMMDD, z.B: '181024'
        lat=48.148: Int/Float mit dem Breitengrad des Orts, Standard: Muenchen 
        lon=11.573: Int/Float mit dem Laengengrad des Orts, Standard: Muenchen
        elev=515: Integer mit der Hoehe des Orts ueber NN, Standard: Muenchen
        ret_format='str': String mit der Information, in welchem Format der Sonnenaufgang und -untergang zurückgegeben werden soll ('str' als String im Format HHMM, 'datetime' als datetime-Objekt)
    """
    import ephem as eph #Module for Sunrise and sunset calculation
    location=eph.Observer()
    location.lat=str(lat)
    location.lon=str(lon)
    location.elevation=elev
    location.date=eph.Date('20'+datum[0:2]+'/'+datum[2:4]+'/'+datum[4:6])
    sundata= eph.Sun()
    sr=location.next_rising(sundata)
    sus=location.next_setting(sundata)
    srform=sr.datetime()
    susform=sus.datetime()
    if ret_format=='str':
        sunrise=srform.strftime('%H%M')
        sunset=susform.strftime('%H%M')
        return sunrise, sunset
    elif ret_format=='datetime':
        return srform, susform
    else:
        raise ValueError 

def getData(filelocation):
    """
    Checks, if the data is available and adds a .gz to the location, when needed
    Arguments:
        filelocation: original location of the file as string, e.g. "/home/.../devicename/data/***.nc"
    """
    if os.path.exists(filelocation)==False:
        if os.path.exists(filelocation+'.gz'):
            print('Decompression needed. This will take some extra time and creates a temp.nc-file in script-folder!')
            with gzip.open(filelocation+'.gz', 'rb') as f_in, open('temp.nc', 'wb') as f_out:
                shutil.copyfileobj(f_in, f_out)
            dataset=data('temp.nc','r')
        else:
            print('Error: Dataset not found, location: '+filelocation)
    else:
        dataset=data(filelocation,'r',format='NETCDF4')
    return dataset

def create_const_time_grid(raw_data,time,date_form,ts,n_ts,factor=1,shift=0):
    """
    Creates a new matrix with constant time grid
    Arguments:
        raw_data: Numpy matrix with an inconsistent time grid, e.g. cloud radar data
        time: array with time data of the measurements
        date_form: date of the day as a datetime object as starting point
        ts: time difference between each column in the matrix with the const time grid as float, unit: seconds, e.g: 10.8
        n_ts: Number of timesteps -> column number new matrix
        factor=1: Usually no data is filled with ones, but here you can multiply the ones with a factor 
        shift=0: You can select any number for no data, which you want, if you shift to it 
    """
    import datetime as dt
    dts=np.empty((n_ts),dtype=object)
    for i in range(n_ts):
        dts[i]=date_form+dt.timedelta(seconds=ts*i)
    gridded_data=np.ones((raw_data.shape[0],n_ts))*factor+shift
    x_nodata=[]
    nr=0
    area=[0,0]
    for no in range(0,n_ts):
        if nr==raw_data.shape[1]:
            x_nodata.append(no)
            continue
        diff=(time[nr]-dts[no]).total_seconds()/ts
        while diff>=-0.5 and diff<0.5 and nr<raw_data.shape[1]:
            nr+=1
            if nr==raw_data.shape[1]:
                area[1]=nr
                gridded_data[:,no]=np.mean(raw_data[:,area[0]:area[1]],axis=1)
                break
            diff=(time[nr]-dts[no]).total_seconds()/ts
        if diff>=0.5:
            if no==n_ts-1:
                area[1]=raw_data.shape[1]
            else:
                area[1]=nr
            if area[0]==area[1]:
                x_nodata.append(no)
                continue
            gridded_data[:,no]=np.mean(raw_data[:,area[0]:area[1]],axis=1)
            #print area, no;
            area[0]=nr
    return gridded_data, x_nodata,dts

def fill_gaps(raw_data,nodata):
    """
     Fills small gaps of 1 data point with averaged data
     Arguments:
         raw_data: Numpy matrix with small gaps, e.g. Hatpro Data
         nodata: array with the addresses of all missing data
    """
    filled_data=1*raw_data
    if type(nodata)==list:
        nodata=np.array(nodata)
    for i in range(nodata.shape[0]):
        if i>0 and i<nodata.shape[0]-1:
            if nodata[i]+1 in nodata or  nodata[i]-1 in nodata:
                continue
            filled_data[:,nodata[i]]=(filled_data[:,nodata[i]+1]+filled_data[:,nodata[i]-1])/2.
    return filled_data
