# -----------------------------------------------------------------------------
# wesely, exo_coldens
# -----------------------------------------------------------------------------

ln -s ${WRFmztoolsdir}/* .

wesely < wesely.inp > wesely.out

exo_coldens < exo_coldens.inp > exo_coldens.out

# -----------------------------------------------------------------------------
# MEGAN
# -----------------------------------------------------------------------------

if [ -f megan_bio_emiss.inp ]
then

# MEGAN takes a long time, but does not really depend on the time of simulation.
# Hence we create one file for the whole year (that is - LAI for all months)
# and only replace start date etc ...

# in case the namelist.wps is newer (i.e., probably the domain setup has changed, we need to
# recreate the files
if [ ${chainDir}/blueprints/${experiment}/namelist.wps -nt ${workDir}/${experiment}/wrfbiochemi_d01 ]
then
  rm -f ${workDir}/${experiment}/wrfbiochemi_d0[0-9]
fi

# assume if one is missing, all are missing!
if [ ! -f ${workDir}/${experiment}/wrfbiochemi_d01 ]
then
  echo "MEGAN input files do not not exist - recreating from scratch. Grab a coffee..."
  ln -s ${WRFMEGANdir}/megan_bio_emiss .
  megan_bio_emiss < megan_bio_emiss.inp > megan_bio_emiss.out
  for domain in $(seq -f "0%g" 1 ${max_dom})
  do
    meganDataFile=${workDir}/${experiment}/wrfbiochemi_d${domain}
    mv wrfbiochemi_d${domain} ${meganDataFile}
  done
fi

julday=$(date -u --date="__refDateTxt__" '+%-j')
for domain in $(seq -f "0%g" 1 ${max_dom})
do
  meganDataFile=${workDir}/${experiment}/wrfbiochemi_d${domain}
  rm -f wrfbiochemi_d${domain}
  ncatted -a START_DATE,global,o,c,"__refDate__" \
          -a SIMULATION_START_DATE,global,o,c,"__refDate__" \
          -a JULYR,global,o,i,__refYear__ \
          -a JULDAY,global,o,i,${julday} \
          ${meganDataFile} wrfbiochemi_d${domain}
  ncks -A -v Times wrfinput_d${domain} wrfbiochemi_d${domain}
done
fi

# -----------------------------------------------------------------------------
# TUV (>v4)
# -----------------------------------------------------------------------------

if [ -f ${TUVDataPath} ]
then
  tar xvjf ${TUVDataPath} > tuv_extract.log
fi

# -----------------------------------------------------------------------------
# emissions (anthropogenic / gridded)
# -----------------------------------------------------------------------------

if [ "$emissUseAnthroEmiss" = "true" ]
then
  if [ -f anthro_emis.inp ]
  then
    # using anthro_emiss
    ln -s ${WRFanthrodir}/anthro_emis .
    # difference in years for emission offset
    emissYearOffset=0
    let "emissYearOffset = __startYear__ - emissYear"
    sed -e "s/__emissYearOffset__/${emissYearOffset}/g" -i anthro_emis.inp
    anthro_emis < anthro_emis.inp > anthro_emis.out
  fi
else
  # preparing existing wrfchemi_files
  for domain in $(seq 1 ${max_dom})
  do
    if [ -f ${emissDir}/wrfchemi_00z_d0${domain} ]
    then
      rm -f wrfchemi_00z_d0${domain} wrfchemi_12z_d0${domain}
      ncks -v XLAT wrfinput_d0${domain} wrfchemi_00z_d0${domain}
      cp wrfchemi_00z_d0${domain} wrfchemi_12z_d0${domain}
      ncks -A ${emissDir}/wrfchemi_00z_d0${domain} wrfchemi_00z_d0${domain}
      ncks -A ${emissDir}/wrfchemi_12z_d0${domain} wrfchemi_12z_d0${domain}
    fi
  done
fi

# apply diurnal cycle
ln -s ${WRFemitdir}/* .
rm -f final_output
cp -rp ${WRFemitdir}/final_output .
ncl MAIN_emission_processing.ncl > diurnal_emiss.out
python sum_sector_emiss_wrfchemi.py >> diurnal_emiss.out

# -----------------------------------------------------------------------------
# fire emissions
# -----------------------------------------------------------------------------

# using fire_emis:
if [ -f fire_emis.inp ]
then
  ln -s ${WRFfiredir}/fire_emis .
  
  # ln -s $(dirname ${fireFilePattern})/*nc .
  
  # # put contents of first day source file into local file "FINN_FIRES.txt"
  # curFile=$(date -u --date="__startYear__-__startMonth__-__startDay__" "+${fireFilePattern}")
  # [ -n "$(ls -1 ${curFile} 2> /dev/null | head -1)" ] || echo "No fire input files found."
  
  # cat < ${curFile} > ./FINN_FIRES.txt

  # # loop over following days - if not found, take last existing one and replace date
  # # reason: no fire files for the future, assume persistence...
  # let totTime="fcstTime+spinupTime+metInc+24"
  # for hour in $(seq -w 24 24 ${totTime})
  # do
  #   nextFile=$(date -u --date="__startYear__-__startMonth__-__startDay__ ${hour} hours" "+${fireFilePattern}")
  #   nextJDay=$(date -u --date="__startYear__-__startMonth__-__startDay__ ${hour} hours" "+%j")
  #   if [ -n "$(ls -1 ${nextFile} 2> /dev/null | head -1)" ]
  #   then
  #     tail -n+2 < ${nextFile} >> ./FINN_FIRES.txt
  #     curFile=${nextFile}
  #   else
  #     echo "${nextFile} fire input expected, but missing. Assuming persistence based on $(basename ${curFile})."
  #     # replace Julian Day in file (first field) with current Julian day
  #     pattern=$(printf "%6d" $((10#${nextJDay})) )
  #     sed -E -s "s/^[ ]*[0-9]+,/${pattern},/g" ${curFile} | tail -n+2 >> ./FINN_FIRES.txt
  #   fi
  # done
  # /bin/sed -s "s/__domain__/${max_dom}/" -i fire_emis.inp
  fire_emis < fire_emis.inp > fire_emis.out
fi

# -----------------------------------------------------------------------------
# second real
# -----------------------------------------------------------------------------

cp -r ${WRFChemDir}/run/* .
rm *.exe
cp -r ${WRFChemDir}/main/*.exe .

cp namelist.wrf.real namelist.input
cp iofields.chem      iofields

${mpiCommandReal} ./real.exe

mkdir -p second_real_out
mv rsl* second_real_out/
cp wrfinput_d01 wrf_chem_input_d01

# -----------------------------------------------------------------------------
# mozbc IC/BC
# -----------------------------------------------------------------------------

if [ -f mozbc_outer.inp ]
then
ln -s $WRFMOZARTdir/mozbc .

# If daily MOZBC files use this portion (otherwise comment out)
# let totDays="((spinupTime+fcstTime)/24)+2"
# i=0
# for day in $(seq -w -1 $totDays)
# do
#   # date format stupidity
#   txty="${day} days"
#   if [ $day -lt 0 ]
#   then
#     txty="${day/-/} days ago"
#   fi

#   curFile=$(date -u --date="__refYear__-__refMonth__-__refDay__ __refHour__:00:00 ${txty}" "+${chembcFilePattern}")

#   ln -s ${curFile} ./moz$(printf "%04d" $i).nc
#   let i=i+1
# done

# If monthly MOZBC files use this portion (otherwise comment out)
# MOZART - pre 2018
ln -s ${MOZARTdir}/MZ2015oct ./moz0000.nc
ln -s ${MOZARTdir}/MZ2015nov ./moz0001.nc
ln -s ${MOZARTdir}/MZ2015dec ./moz0002.nc
# WACCM - post 2018
#ln -s ${MOZARTdir}/WACCM2019jan ./moz0000.nc
#ln -s ${MOZARTdir}/WACCM2019feb ./moz0001.nc
#ln -s ${MOZARTdir}/WACCM2019mar ./moz0002.nc

# first domain, always
mozbc < mozbc_outer.inp > mozbc_bc.log

for domain in $(seq 2 ${max_dom})
do
  /bin/sed -s "s/__domain__/${domain}/" < mozbc_inner.inp > mozbc_inner_d${domain}.inp
  mozbc < mozbc_inner_d${domain}.inp > mozbc_inner_d${domain}.log
done
fi

